#ifndef MOVEMODE_H
#define MOVEMODE_H


enum class MoveMode {
    MIDDLE,
    TOPLEFT,
    TOPRIGHT,
    BOTTOMLEFT,
    BOTTOMRIGHT,
    ROTATION,
    ROTATIONLEFT,
    ROTATIONRIGHT,
    SYMMETRY,
    REPOSITION,
    PERSP_SINGLE,
    PERSP_LEFT,
    PERSP_RIGHT,
    PERSP_MIDDLE,
    CENTER,
    NONE
};

#endif // MOVEMODE_H
